import type { SyntaxHighlighterProps } from 'src/types/shiki'
import type { ReactStyle } from '../../utils/style-transformer'
import React, { useMemo } from 'react'
import { Platform, ScrollView, StyleSheet, Text, type TextStyle, View } from 'react-native'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'
import { getRNStylesFromShikiStyle } from '../../utils/style-transformer'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

const styles = StyleSheet.create({
  scrollView: {
    flex: 1,
    minHeight: 20,
  },
  lineContainer: {
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
})

const SyntaxHighlighter: React.FC<SyntaxHighlighterProps> = ({ text, language, languageId, theme }) => {
  const stylesheet = useMemo(() => getRNStylesFromShikiStyle(theme as ReactStyle), [theme])
  const { tokens } = useSyntaxHighlighter({
    text,
    language,
    languageId,
    theme,
  })

  const renderToken = (token: any, index: number, lineIndex: number) => {
    const tokenStyles: TextStyle = StyleSheet.flatten([
      stylesheet.token,
      {
        fontFamily: monospaceFont,
        ...(token.color && { color: token.color }),
        ...(token.fontStyle === 'italic' && stylesheet.italic),
        ...(token.fontWeight === 'bold' && stylesheet.bold),
      },
    ])

    return (
      <Text key={`${lineIndex}-${index}`} style={tokenStyles}>
        {token.content.replace(/ /g, '\u00A0')}
      </Text>
    )
  }

  const renderLine = (line: any[], lineIndex: number) => (
    <View key={lineIndex} style={styles.lineContainer}>
      {line.map((token, tokenIndex) => renderToken(token, tokenIndex, lineIndex))}
    </View>
  )

  return (
    <ScrollView horizontal showsHorizontalScrollIndicator={Platform.OS !== 'web'} style={styles.scrollView}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((line, index) => renderLine(line, index))}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
